import processing.serial.*;

PFont font;

// Valeurs pour la liaison série : 
Serial serial;
int serialPort = 1; // changer selon la machine

boolean flag_reception = false;
String trame;
String[] parts;

// Valeurs pour la capture : 

int sen = 3; // sensors

int[] valuesX;
int[] valuesY;
int[] valuesZ;

int averageX = 0;
int averageY = 0;
int averageZ = 0;

int ratioX = 10;
int ratioY = 10;
int ratioZ = 10;

// Valeurs pour le chrono :

long temps = 0;

// Valeurs de l'échantillonnage pour moyenne :
int accu_mean = 0;
long accuX = 0;
long accuY = 0;
long accuZ = 0;

// Valeurs pour les calculs d'IMU :

boolean flag_startIMU = false;

int angleX = 0;
int angleY = 0;
int angleZ = 0;


/** 
 * Configuration
 */
void setup() 
{
  size(640, 480,P3D);
  frameRate(25);

  font = loadFont("ArialMT-20.vlw"); 
  textFont(font, 32); 

  // Open the port that the board is connected to and use the same speed (9600 bps)

  println(Serial.list());
  serial = new Serial(this, Serial.list()[serialPort], 115200);  
  serial.bufferUntil(10); // line feed

  valuesX = new int[width];
  valuesY = new int[width];
  valuesZ = new int[width];
  smooth();
}


int getY(int val) {
  return (int)(val / 1023.0f * height) - 1;
}

/**
 * Réception d'une trame
 */
void serialEvent(Serial p) {

  flag_reception = true;

  // juste pour vérifier qu'on arrive à gérer le temps delta
  // println(millis()-temps);
  temps = millis();

  // traiter la chaine de caractères
  trame = p.readString();
  // vérifier la longueur
  if (trame.length() < 15) {
    print("e");
    return;
  }   

  parts = split(trame, " ");

  // décaler les valeurs précédentes  
  for (int i=0; i<width-1; i++) {
    valuesX[i] = valuesX[i+1];
    valuesY[i] = valuesY[i+1];
    valuesZ[i] = valuesZ[i+1];    
  }

  // enregistrer les nouvelles valeurs filtrées
  valuesX[width-1] = int(float(parts[1])) - averageX;
  valuesY[width-1] = int(float(parts[2])) - averageY;
  valuesZ[width-1] = int(float(parts[3])) - averageZ;

  // nouvelle valeur, traiter l'IMU
  if (flag_startIMU) 
  {    
    // 5000 pour 360%
    
    // intégration du signal
    angleX += valuesX[width-1]/100;
    angleY += valuesY[width-1]/100;
    angleZ += valuesZ[width-1]/100;
    
    println("angles: "+angleX*360/5000+" "+angleY*360/5000+" "+angleZ*360/5000);
    

  }


}

void draw()
{ 
  if (flag_reception && accu_mean <= 100) {
    accu_mean++;
    print(".");
    // gestion du démarrage
    if (accu_mean < 50) {
      // ne rien faire les milles premières fois
      return;
    } 
    else if (accu_mean < 100) {      
      accuX +=  valuesX[width-1];
      accuY +=  valuesY[width-1];
      accuZ +=  valuesZ[width-1];
      return ;
    } 
    else {
      averageX = (int) accuX / 50;
      averageY = (int) accuY / 50;
      averageZ = (int) accuZ / 50;   
      println(); 
      println("Moyennes calculées : "+averageX+","+averageY+","+averageZ);
      //
      flag_startIMU = true;
    } 
  } 
  else {   
    updateViewAngles(); 
  }
}

void updateViewAngles() {
  
  background(250);
  fill(250,0,0);
  
  translate(width/2, height/2, 0);
  rotateY(angleX*2*PI/5000);
  rotateX(angleY*2*PI/5000);
  rotateZ(angleZ*2*PI/5000);
  box(150);
  
}


void updateViewOscillo() {

  background(250);

  for (int x=1; x<width; x++) {
    //
    stroke(150,0,0);
    line(width-x,   height-1-getY((valuesX[x-1]/ratioX)+250), 
    width-1-x, height-1-getY((valuesX[x]/ratioX)+250));
    //    
    stroke(0,150,0);
    line(width-x,   height-1-getY((valuesY[x-1]/ratioY)+510), 
    width-1-x, height-1-getY((valuesY[x]/ratioX)+510));
    //
    stroke(0,0,150);
    line(width-x,   height-1-getY((valuesZ[x-1]/ratioZ)+750), 
    width-1-x, height-1-getY((valuesZ[x]/ratioX)+750));
  }

  // afficher la valeur courante  
  fill(0);
  text(valuesX[width-1]/30, width-100, getY(350));
  text(valuesY[width-1]/30, width-100, getY(600));
  text(valuesZ[width-1]/30, width-100, getY(850));
  
 

}















